@@ -12,15 +12,12 @@ from .models.subsegment import Subsegment, SubsegmentContextManager
 from .models.default_dynamic_naming import DefaultDynamicNaming
 from .models.dummy_entities import DummySegment, DummySubsegment
 from .emitters.udp_emitter import UDPEmitter
-from .sampling.sampler import DefaultSampler
-from .sampling.local.sampler import LocalSampler
 from .streaming.default_streaming import DefaultStreaming
 from .context import Context
 from .daemon_config import DaemonConfig
 from .plugins.utils import get_plugin_modules
 from .lambda_launcher import check_in_lambda
 from .exceptions.exceptions import SegmentNameMissingException, SegmentNotFoundException
-from .utils.compat import string_types
 from .utils import stacktrace
 
 log = logging.getLogger(__name__)
@@ -42,7 +39,7 @@ SERVICE_INFO = {
 }
 
 
-class AWSXRayRecorder(object):
+class AWSXRayRecorder:
     """
     A global AWS X-Ray recorder that will begin/end segments/subsegments
     and send them to the X-Ray daemon. This recorder is initialized during
@@ -58,10 +55,12 @@ class AWSXRayRecorder(object):
         context = check_in_lambda()
         if context:
             # Special handling when running on AWS Lambda.
+            from .sampling.local.sampler import LocalSampler
             self._context = context
             self.streaming_threshold = 0
             self._sampler = LocalSampler()
         else:
+            from .sampling.sampler import DefaultSampler
             self._context = Context()
             self._sampler = DefaultSampler()
 
@@ -89,6 +88,7 @@ class AWSXRayRecorder(object):
 
         Configure needs to run before patching thrid party libraries
         to avoid creating dangling subsegment.
+
         :param bool sampling: If sampling is enabled, every time the recorder
             creates a segment it decides whether to send this segment to
             the X-Ray daemon. This setting is not used if the recorder
@@ -112,6 +112,7 @@ class AWSXRayRecorder(object):
             RUNTIME_ERROR means the recorder will raise an exception.
             LOG_ERROR means the recorder will only log the error and
             do nothing.
+            IGNORE_ERROR means the recorder will do nothing
         :param str daemon_address: The X-Ray daemon address where the recorder
             sends data to.
         :param str service: default segment name if creating a segment without
@@ -184,7 +185,7 @@ class AWSXRayRecorder(object):
 
     def in_segment(self, name=None, **segment_kwargs):
         """
-        Return a segment context manger.
+        Return a segment context manager.
 
         :param str name: the name of the segment
         :param dict segment_kwargs: remaining arguments passed directly to `begin_segment`
@@ -193,7 +194,7 @@ class AWSXRayRecorder(object):
 
     def in_subsegment(self, name=None, **subsegment_kwargs):
         """
-        Return a subsegment context manger.
+        Return a subsegment context manager.
 
         :param str name: the name of the subsegment
         :param dict subsegment_kwargs: remaining arguments passed directly to `begin_subsegment`
@@ -231,7 +232,7 @@ class AWSXRayRecorder(object):
         elif sampling:
             decision = sampling
         elif self.sampling:
-            decision = self._sampler.should_trace()
+            decision = self._sampler.should_trace({'service': seg_name})
 
         if not decision:
             segment = DummySegment(seg_name)
@@ -273,16 +274,10 @@ class AWSXRayRecorder(object):
         else:
             return entity
 
-    def begin_subsegment(self, name, namespace='local'):
-        """
-        Begin a new subsegment.
-        If there is open subsegment, the newly created subsegment will be the
-        child of latest opened subsegment.
-        If not, it will be the child of the current open segment.
-
-        :param str name: the name of the subsegment.
-        :param str namespace: currently can only be 'local', 'remote', 'aws'.
-        """
+    def _begin_subsegment_helper(self, name, namespace='local', beginWithoutSampling=False):
+        '''
+        Helper method to begin_subsegment and begin_subsegment_without_sampling
+        '''
         # Generating the parent dummy segment is necessary.
         # We don't need to store anything in context. Assumption here
         # is that we only work with recorder-level APIs.
@@ -294,15 +289,41 @@ class AWSXRayRecorder(object):
             log.warning("No segment found, cannot begin subsegment %s." % name)
             return None
 
-        if not segment.sampled:
+        current_entity = self.get_trace_entity()
+        if not current_entity.sampled or beginWithoutSampling:
             subsegment = DummySubsegment(segment, name)
         else:
             subsegment = Subsegment(name, namespace, segment)
 
         self.context.put_subsegment(subsegment)
-
         return subsegment
 
+
+
+    def begin_subsegment(self, name, namespace='local'):
+        """
+        Begin a new subsegment.
+        If there is open subsegment, the newly created subsegment will be the
+        child of latest opened subsegment.
+        If not, it will be the child of the current open segment.
+
+        :param str name: the name of the subsegment.
+        :param str namespace: currently can only be 'local', 'remote', 'aws'.
+        """
+        return self._begin_subsegment_helper(name, namespace)
+
+
+    def begin_subsegment_without_sampling(self, name):
+        """
+        Begin a new unsampled subsegment.
+        If there is open subsegment, the newly created subsegment will be the
+        child of latest opened subsegment.
+        If not, it will be the child of the current open segment.
+
+        :param str name: the name of the subsegment.
+        """
+        return self._begin_subsegment_helper(name, beginWithoutSampling=True)
+
     def current_subsegment(self):
         """
         Return the latest opened subsegment. In a multithreading environment,
@@ -465,7 +486,7 @@ class AWSXRayRecorder(object):
         segment.set_aws(copy.deepcopy(self._aws_metadata))
         segment.set_service(SERVICE_INFO)
 
-        if isinstance(sampling_decision, string_types):
+        if isinstance(sampling_decision, str):
             segment.set_rule_name(sampling_decision)
 
     def _send_segment(self):
@@ -485,7 +506,8 @@ class AWSXRayRecorder(object):
 
     def _stream_subsegment_out(self, subsegment):
         log.debug("streaming subsegments...")
-        self.emitter.send_entity(subsegment)
+        if subsegment.sampled:
+            self.emitter.send_entity(subsegment)
 
     def _load_sampling_rules(self, sampling_rules):
 
@@ -540,7 +562,7 @@ class AWSXRayRecorder(object):
 
     @dynamic_naming.setter
     def dynamic_naming(self, value):
-        if isinstance(value, string_types):
+        if isinstance(value, str):
             self._dynamic_naming = DefaultDynamicNaming(value, self.service)
         else:
             self._dynamic_naming = value
